@@ -190,7 +190,9 @@ class Field(ToJson, FromJson["Field"]):
 
 
 class Document(ToJson, FromJson["Document"]):
-    def __init__(self, fields: Optional[List[Field]] = None) -> None:
+    def __init__(
+        self, fields: Optional[List[Field]] = None, inherits: Optional[str] = None
+    ) -> None:
         """
         Create a Vespa Document.
 
@@ -202,12 +204,15 @@ class Document(ToJson, FromJson["Document"]):
         To create a Document:
 
         >>> Document()
-        Document(None)
+        Document(None, None)
 
         >>> Document(fields=[Field(name="title", type="string")])
-        Document([Field('title', 'string', None, None, None, None)])
+        Document([Field('title', 'string', None, None, None, None)], None)
 
+        >>> Document(fields=[Field(name="title", type="string")], inherits="context")
+        Document([Field('title', 'string', None, None, None, None)], context)
         """
+        self.inherits = inherits
         self._fields = (
             OrderedDict()
             if not fields
@@ -230,21 +235,29 @@ class Document(ToJson, FromJson["Document"]):
 
     @staticmethod
     def from_dict(mapping: Mapping) -> "Document":
-        return Document(fields=[FromJson.map(field) for field in mapping.get("fields")])
+        return Document(
+            fields=[FromJson.map(field) for field in mapping.get("fields")],
+            inherits=mapping.get("inherits", None),
+        )
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"fields": [field.to_envelope for field in self.fields]}
+        map = {
+            "fields": [field.to_envelope for field in self.fields],
+            "inherits": self.inherits,
+        }
         return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
-        return self.fields == other.fields
+        return self.fields == other.fields and self.inherits == other.inherits
 
     def __repr__(self):
-        return "{0}({1})".format(
-            self.__class__.__name__, repr(self.fields) if self.fields else None
+        return "{0}({1}, {2})".format(
+            self.__class__.__name__,
+            repr(self.fields) if self.fields else None,
+            self.inherits,
         )
 
 
@@ -666,7 +679,7 @@ class Schema(ToJson, FromJson["Schema"]):
         To create a Schema:
 
         >>> Schema(name="schema_name", document=Document())
-        Schema('schema_name', Document(None), None, None, [])
+        Schema('schema_name', Document(None, None), None, None, [])
         """
         self.name = name
         self.document = document
@@ -716,6 +729,28 @@ class Schema(ToJson, FromJson["Schema"]):
         """
         self.models.append(model)
 
+    @property
+    def schema_to_text(self):
+        env = Environment(
+            loader=PackageLoader("vespa", "templates"),
+            autoescape=select_autoescape(
+                disabled_extensions=("txt",),
+                default_for_string=True,
+                default=True,
+            ),
+        )
+        env.trim_blocks = True
+        env.lstrip_blocks = True
+        schema_template = env.get_template("schema.txt")
+        return schema_template.render(
+            schema_name=self.name,
+            document_name=self.name,
+            document=self.document,
+            fieldsets=self.fieldsets,
+            rank_profiles=self.rank_profiles,
+            models=self.models,
+        )
+
     @staticmethod
     def from_dict(mapping: Mapping) -> "Schema":
         fieldsets = mapping.get("fieldsets", None)
@@ -994,7 +1029,7 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
     def __init__(
         self,
         name: str,
-        schema: Optional[Schema] = None,
+        schema: Optional[List[Schema]] = None,
         query_profile: Optional[QueryProfile] = None,
         query_profile_type: Optional[QueryProfileType] = None,
     ) -> None:
@@ -1005,7 +1040,7 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         for more detailed information about application packages.
 
         :param name: Application name.
-        :param schema: :class:`Schema` of the application. If `None`, an empty :class:`Schema` with the same name of the
+        :param schema: List of :class:`Schema`s of the application. If `None`, an empty :class:`Schema` with the same name of the
             application will be created by default.
         :param query_profile: :class:`QueryProfile` of the application. If `None`, a :class:`QueryProfile` named `default`
          with :class:`QueryProfileType` named `root` will be created by default.
@@ -1015,15 +1050,15 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         The easiest way to get started is to create a default application package:
 
         >>> ApplicationPackage(name="test_app")
-        ApplicationPackage('test_app', Schema('test_app', Document(None), None, None, []), QueryProfile(None), QueryProfileType(None))
+        ApplicationPackage('test_app', [Schema('test_app', Document(None, None), None, None, [])], QueryProfile(None), QueryProfileType(None))
 
         It will create a default :class:`Schema`, :class:`QueryProfile` and :class:`QueryProfileType` that you can then
         populate with specifics of your application.
         """
         self.name = name
         if not schema:
-            schema = Schema(name=self.name, document=Document())
-        self.schema = schema
+            schema = [Schema(name=self.name, document=Document())]
+        self._schema = OrderedDict([(x.name, x) for x in schema])
         if not query_profile:
             query_profile = QueryProfile()
         self.query_profile = query_profile
@@ -1033,6 +1068,30 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         self.model_ids = []
         self.model_configs = {}
 
+    @property
+    def schemas(self) -> List[Schema]:
+        return [x for x in self._schema.values()]
+
+    @property
+    def schema(self):
+        assert (
+            len(self.schemas) == 1
+        ), "Your application has more than one Schema, use get_schema instead."
+        return self.schemas[0]
+
+    def get_schema(self, name):
+        return self._schema[name]
+
+    def add_schema(self, *schemas: Schema) -> None:
+        """
+        Add :class:`Schema`'s to the application package.
+
+        :param schemas: schemas to be added
+        :return:
+        """
+        for schema in schemas:
+            self._schema.update({schema.name: schema})
+
     def add_model_ranking(
         self, model_config: ModelConfig, include_model_summary_features=False, **kwargs
     ) -> None:
@@ -1203,28 +1262,6 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
             )
         )
 
-    @property
-    def schema_to_text(self):
-        env = Environment(
-            loader=PackageLoader("vespa", "templates"),
-            autoescape=select_autoescape(
-                disabled_extensions=("txt",),
-                default_for_string=True,
-                default=True,
-            ),
-        )
-        env.trim_blocks = True
-        env.lstrip_blocks = True
-        schema_template = env.get_template("schema.txt")
-        return schema_template.render(
-            schema_name=self.schema.name,
-            document_name=self.schema.name,
-            fields=self.schema.document.fields,
-            fieldsets=self.schema.fieldsets,
-            rank_profiles=self.schema.rank_profiles,
-            models=self.schema.models,
-        )
-
     @property
     def query_profile_to_text(self):
         env = Environment(
@@ -1285,45 +1322,62 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         schema_template = env.get_template("services.xml")
         return schema_template.render(
             application_name=self.name,
-            document_name=self.schema.name,
+            schemas=self.schemas,
         )
 
     @staticmethod
     def from_dict(mapping: Mapping) -> "ApplicationPackage":
         schema = mapping.get("schema", None)
         if schema is not None:
-            schema = FromJson.map(schema)
+            schema = [FromJson.map(x) for x in schema]
         return ApplicationPackage(name=mapping["name"], schema=schema)
 
     @property
     def to_dict(self) -> Mapping:
         map = {"name": self.name}
-        if self.schema is not None:
-            map.update({"schema": self.schema.to_envelope})
+        if self._schema is not None:
+            map.update({"schema": [x.to_envelope for x in self.schemas]})
         return map
 
+    @staticmethod
+    def _application_package_file_name(disk_folder):
+        return os.path.join(disk_folder, "application_package.json")
+
+    def save(self, disk_folder: str) -> None:
+        Path(disk_folder).mkdir(parents=True, exist_ok=True)
+        file_path = ApplicationPackage._application_package_file_name(disk_folder)
+        with open(file_path, "w") as f:
+            f.write(self.to_json)
+
+    @staticmethod
+    def load(disk_folder: str) -> "ApplicationPackage":
+        file_path = ApplicationPackage._application_package_file_name(disk_folder)
+        with open(file_path, "r") as f:
+            return ApplicationPackage.from_json(f.read())
+
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
-        return self.name == other.name and self.schema == other.schema
+        return self.name == other.name and self._schema == other._schema
 
     def __repr__(self):
         return "{0}({1}, {2}, {3}, {4})".format(
             self.__class__.__name__,
             repr(self.name),
-            repr(self.schema),
+            repr(self.schemas),
             repr(self.query_profile),
             repr(self.query_profile_type),
         )
 
 
-class VespaDocker(object):
+class VespaDocker(ToJson, FromJson["VespaDocker"]):
     def __init__(
         self,
         disk_folder: str,
         port: int = 8080,
-        container_memory: str = "4G",
+        container_memory: Union[str, int] = 4 * (1024 ** 3),
         output_file: IO = sys.stdout,
+        container: Optional[docker.models.containers.Container] = None,
     ) -> None:
         """
         Manage Docker deployments.
@@ -1331,14 +1385,52 @@ class VespaDocker(object):
         :param port: Container port.
         :param output_file: Output file to write output messages.
         :param container_memory: Docker container memory available to the application.
+        :param container: Used when instantiating VespaDocker from a running container.
         """
-        self.container = None
+        self.container = container
+        container_id = None
+        container_name = None
+        if container:
+            container_id = container.id
+            container_name = container.name
+        self.container_name = container_name
+        self.container_id = container_id
         self.url = "http://localhost"
         self.local_port = port
         self.disk_folder = disk_folder
         self.container_memory = container_memory
         self.output = output_file
 
+    @staticmethod
+    def from_container_name_or_id(
+        name_or_id: str, output_file: IO = sys.stdout
+    ) -> "VespaDocker":
+        """
+        Instantiate VespaDocker from a running container.
+
+        :param name_or_id: Name or id of the container.
+        :param output_file: Output file to write output messages.
+        :return: VespaDocker instance associated with the running container.
+        """
+        client = docker.from_env()
+        try:
+            container = client.containers.get(name_or_id)
+        except docker.errors.NotFound:
+            raise ValueError("The container does not exist.")
+        disk_folder = container.attrs["Mounts"][0]["Source"]
+        port = int(
+            container.attrs["HostConfig"]["PortBindings"]["8080/tcp"][0]["HostPort"]
+        )
+        container_memory = container.attrs["HostConfig"]["Memory"]
+
+        return VespaDocker(
+            disk_folder=disk_folder,
+            port=port,
+            container_memory=container_memory,
+            output_file=output_file,
+            container=container,
+        )
+
     def _run_vespa_engine_container(
         self,
         application_name: str,
@@ -1361,6 +1453,8 @@ class VespaDocker(object):
                     volumes={disk_folder: {"bind": "/app", "mode": "rw"}},
                     ports={8080: self.local_port},
                 )
+            self.container_name = self.container.name
+            self.container_id = self.container.id
         else:
             self.container.restart()
 
@@ -1390,14 +1484,26 @@ class VespaDocker(object):
         Path(os.path.join(self.disk_folder, "application/schemas")).mkdir(
             parents=True, exist_ok=True
         )
-        with open(
-            os.path.join(
-                self.disk_folder,
-                "application/schemas/{}.sd".format(application_package.schema.name),
-            ),
-            "w",
-        ) as f:
-            f.write(application_package.schema_to_text)
+        Path(os.path.join(self.disk_folder, "application/files")).mkdir(
+            parents=True, exist_ok=True
+        )
+
+        for schema in application_package.schemas:
+            with open(
+                os.path.join(
+                    self.disk_folder,
+                    "application/schemas/{}.sd".format(schema.name),
+                ),
+                "w",
+            ) as f:
+                f.write(schema.schema_to_text)
+            for model in schema.models:
+                copyfile(
+                    model.model_file_path,
+                    os.path.join(
+                        self.disk_folder, "application/files", model.model_file_name
+                    ),
+                )
 
         Path(
             os.path.join(self.disk_folder, "application/search/query-profiles/types")
@@ -1423,17 +1529,6 @@ class VespaDocker(object):
         with open(os.path.join(self.disk_folder, "application/services.xml"), "w") as f:
             f.write(application_package.services_to_text)
 
-        Path(os.path.join(self.disk_folder, "application/files")).mkdir(
-            parents=True, exist_ok=True
-        )
-        for model in application_package.schema.models:
-            copyfile(
-                model.model_file_path,
-                os.path.join(
-                    self.disk_folder, "application/files", model.model_file_name
-                ),
-            )
-
     def _execute_deployment(
         self,
         application_name: str,
@@ -1490,9 +1585,7 @@ class VespaDocker(object):
         :return: a Vespa connection instance.
         """
 
-        self.export_application_package(
-            application_package=application_package
-        )
+        self.export_application_package(application_package=application_package)
 
         return self._execute_deployment(
             application_name=application_package.name,
@@ -1504,13 +1597,11 @@ class VespaDocker(object):
     def deploy_from_disk(
         self,
         application_name: str,
-        disk_folder: str,
         application_folder: Optional[str] = None,
     ) -> Vespa:
         """
         Deploy disk-based application package into a Vespa container.
         :param application_name: Name of the application.
-        :param disk_folder: The disk_folder will be mapped to the /app directory inside the Docker container.
         :param application_folder: The folder inside `disk_folder` containing the application files. If None,
             we assume `disk_folder` to be the application folder.
         :return: a Vespa connection instance.
@@ -1518,7 +1609,7 @@ class VespaDocker(object):
 
         return self._execute_deployment(
             application_name=application_name,
-            disk_folder=disk_folder,
+            disk_folder=self.disk_folder,
             container_memory=self.container_memory,
             application_folder=application_folder,
         )
@@ -1582,6 +1673,69 @@ class VespaDocker(object):
         self.stop_services()
         self.start_services()
 
+    @staticmethod
+    def from_dict(mapping: Mapping) -> "VespaDocker":
+        try:
+            if mapping["container_id"] is not None:
+                vespa_docker = VespaDocker.from_container_name_or_id(
+                    name_or_id=mapping["container_id"]
+                )
+                return vespa_docker
+            elif mapping["container_name"] is not None:
+                vespa_docker = VespaDocker.from_container_name_or_id(
+                    name_or_id=mapping["container_name"]
+                )
+                return vespa_docker
+            else:
+                print(
+                    "Unable to instantiate VespaDocker from a running container. Starting new container."
+                )
+        except ValueError:
+            print(
+                "Unable to instantiate VespaDocker from a running container. Starting new container."
+            )
+        vespa_docker = VespaDocker(
+            disk_folder=mapping["disk_folder"],
+            port=mapping["port"],
+            container_memory=mapping["container_memory"],
+        )
+        return vespa_docker
+
+    @property
+    def to_dict(self) -> Mapping:
+        map = {
+            "container_id": self.container_id,
+            "container_name": self.container_name,
+            "url": self.url,
+            "port": self.local_port,
+            "disk_folder": self.disk_folder,
+            "container_memory": self.container_memory,
+        }
+        return map
+
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+        return (
+            self.container_id == other.container_id
+            and self.container_name == other.container_name
+            and self.url == other.url
+            and self.local_port == other.local_port
+            and self.disk_folder == other.disk_folder
+            and self.container_memory == other.container_memory
+        )
+
+    def __repr__(self):
+        return "{0}({1}, {2}, {3}, {4}, {5}, {6})".format(
+            self.__class__.__name__,
+            repr(self.disk_folder),
+            repr(self.url),
+            repr(self.local_port),
+            repr(self.container_name),
+            repr(self.container_id),
+            repr(self.container_memory),
+        )
+
 
 class VespaCloud(object):
     def __init__(
@@ -1735,12 +1889,18 @@ class VespaCloud(object):
     def _to_application_zip(self) -> BytesIO:
         buffer = BytesIO()
         with zipfile.ZipFile(buffer, "a") as zip_archive:
-            zip_archive.writestr(
-                "application/schemas/{}.sd".format(
-                    self.application_package.schema.name
-                ),
-                self.application_package.schema_to_text,
-            )
+
+            for schema in self.application_package.schemas:
+                zip_archive.writestr(
+                    "application/schemas/{}.sd".format(schema.name),
+                    schema.schema_to_text,
+                )
+                for model in schema.models:
+                    zip_archive.write(
+                        model.model_file_path,
+                        os.path.join("application/files", model.model_file_name),
+                    )
+
             zip_archive.writestr(
                 "application/search/query-profiles/default.xml",
                 self.application_package.query_profile_to_text,
@@ -1756,11 +1916,6 @@ class VespaCloud(object):
                 "application/security/clients.pem",
                 self.data_certificate.public_bytes(serialization.Encoding.PEM),
             )
-            for model in self.application_package.schema.models:
-                zip_archive.write(
-                    model.model_file_path,
-                    os.path.join("application/files", model.model_file_name),
-                )
 
         return buffer
 
