@@ -1,4 +1,5 @@
 import unittest, os
+from shutil import rmtree
 
 from vespa.package import (
     HNSW,
@@ -80,16 +81,16 @@ class TestDocument(unittest.TestCase):
     def test_empty_document(self):
         document = Document()
         self.assertEqual(document.fields, [])
-        self.assertEqual(document.to_dict, {"fields": []})
+        self.assertEqual(document.to_dict, {"fields": [], "inherits": None})
         self.assertEqual(document, Document.from_dict(document.to_dict))
 
     def test_document_one_field(self):
-        document = Document()
+        document = Document(inherits="context")
         field = Field(name="test_name", type="string")
         document.add_fields(field)
         self.assertEqual(document.fields, [field])
         self.assertEqual(document, Document.from_dict(document.to_dict))
-        self.assertEqual(document, Document([field]))
+        self.assertEqual(document, Document([field], "context"))
 
     def test_document_two_fields(self):
         document = Document()
@@ -434,6 +435,7 @@ class TestApplicationPackage(unittest.TestCase):
         test_schema = Schema(
             name="msmarco",
             document=Document(
+                inherits="context",
                 fields=[
                     Field(name="id", type="string", indexing=["attribute", "summary"]),
                     Field(
@@ -549,7 +551,7 @@ class TestApplicationPackage(unittest.TestCase):
         )
         self.app_package = ApplicationPackage(
             name="test_app",
-            schema=test_schema,
+            schema=[test_schema],
             query_profile=test_query_profile,
             query_profile_type=test_query_profile_type,
         )
@@ -562,7 +564,7 @@ class TestApplicationPackage(unittest.TestCase):
     def test_schema_to_text(self):
         expected_result = (
             "schema msmarco {\n"
-            "    document msmarco {\n"
+            "    document msmarco inherits context {\n"
             "        field id type string {\n"
             "            indexing: attribute | summary\n"
             "        }\n"
@@ -667,7 +669,7 @@ class TestApplicationPackage(unittest.TestCase):
             "    }\n"
             "}"
         )
-        self.assertEqual(self.app_package.schema_to_text, expected_result)
+        self.assertEqual(self.app_package.schema.schema_to_text, expected_result)
 
     def test_hosts_to_text(self):
         expected_result = (
@@ -722,6 +724,86 @@ class TestApplicationPackage(unittest.TestCase):
         self.assertEqual(self.app_package.query_profile_type_to_text, expected_result)
 
 
+class TestApplicationPackageMultipleSchema(unittest.TestCase):
+    def setUp(self) -> None:
+        news_schema = Schema(
+            name="news",
+            document=Document(
+                fields=[
+                    Field(
+                        name="news_id", type="string", indexing=["attribute", "summary"]
+                    ),
+                ]
+            ),
+        )
+        user_schema = Schema(
+            name="user",
+            document=Document(
+                fields=[
+                    Field(
+                        name="user_id", type="string", indexing=["attribute", "summary"]
+                    ),
+                ]
+            ),
+        )
+        self.app_package = ApplicationPackage(
+            name="test_app",
+            schema=[news_schema, user_schema],
+        )
+
+    def test_application_package(self):
+        self.assertEqual(
+            self.app_package, ApplicationPackage.from_dict(self.app_package.to_dict)
+        )
+
+    def test_schema_to_text(self):
+        expected_news_result = (
+            "schema news {\n"
+            "    document news {\n"
+            "        field news_id type string {\n"
+            "            indexing: attribute | summary\n"
+            "        }\n"
+            "    }\n"
+            "}"
+        )
+        expected_user_result = (
+            "schema user {\n"
+            "    document user {\n"
+            "        field user_id type string {\n"
+            "            indexing: attribute | summary\n"
+            "        }\n"
+            "    }\n"
+            "}"
+        )
+        expected_results = {"news": expected_news_result, "user": expected_user_result}
+
+        for schema in self.app_package.schemas:
+            self.assertEqual(schema.schema_to_text, expected_results[schema.name])
+
+    def test_services_to_text(self):
+        expected_result = (
+            '<?xml version="1.0" encoding="UTF-8"?>\n'
+            '<services version="1.0">\n'
+            '    <container id="test_app_container" version="1.0">\n'
+            "        <search></search>\n"
+            "        <document-api></document-api>\n"
+            "    </container>\n"
+            '    <content id="test_app_content" version="1.0">\n'
+            '        <redundancy reply-after="1">1</redundancy>\n'
+            "        <documents>\n"
+            '            <document type="news" mode="index"></document>\n'
+            '            <document type="user" mode="index"></document>\n'
+            "        </documents>\n"
+            "        <nodes>\n"
+            '            <node distribution-key="0" hostalias="node1"></node>\n'
+            "        </nodes>\n"
+            "    </content>\n"
+            "</services>"
+        )
+
+        self.assertEqual(self.app_package.services_to_text, expected_result)
+
+
 class TestSimplifiedApplicationPackage(unittest.TestCase):
     def setUp(self) -> None:
         self.app_package = ApplicationPackage(name="test_app")
@@ -826,7 +908,7 @@ class TestSimplifiedApplicationPackage(unittest.TestCase):
             "    }\n"
             "}"
         )
-        self.assertEqual(self.app_package.schema_to_text, expected_result)
+        self.assertEqual(self.app_package.schema.schema_to_text, expected_result)
 
     def test_hosts_to_text(self):
         expected_result = (
@@ -881,6 +963,80 @@ class TestSimplifiedApplicationPackage(unittest.TestCase):
         self.assertEqual(self.app_package.query_profile_type_to_text, expected_result)
 
 
+class TestSimplifiedApplicationPackageWithMultipleSchemas(unittest.TestCase):
+    def setUp(self) -> None:
+        self.app_package = ApplicationPackage(name="news")
+
+        self.app_package.schema.add_fields(
+            Field(name="news_id", type="string", indexing=["attribute", "summary"]),
+        )
+        self.app_package.add_schema(
+            Schema(
+                name="user",
+                document=Document(
+                    fields=[
+                        Field(
+                            name="user_id",
+                            type="string",
+                            indexing=["attribute", "summary"],
+                        )
+                    ]
+                ),
+            )
+        )
+
+    def test_application_package(self):
+        self.assertEqual(
+            self.app_package, ApplicationPackage.from_dict(self.app_package.to_dict)
+        )
+
+    def test_schema_to_text(self):
+        expected_news_result = (
+            "schema news {\n"
+            "    document news {\n"
+            "        field news_id type string {\n"
+            "            indexing: attribute | summary\n"
+            "        }\n"
+            "    }\n"
+            "}"
+        )
+        expected_user_result = (
+            "schema user {\n"
+            "    document user {\n"
+            "        field user_id type string {\n"
+            "            indexing: attribute | summary\n"
+            "        }\n"
+            "    }\n"
+            "}"
+        )
+        expected_results = {"news": expected_news_result, "user": expected_user_result}
+
+        for schema in self.app_package.schemas:
+            self.assertEqual(schema.schema_to_text, expected_results[schema.name])
+
+    def test_services_to_text(self):
+        expected_result = (
+            '<?xml version="1.0" encoding="UTF-8"?>\n'
+            '<services version="1.0">\n'
+            '    <container id="news_container" version="1.0">\n'
+            "        <search></search>\n"
+            "        <document-api></document-api>\n"
+            "    </container>\n"
+            '    <content id="news_content" version="1.0">\n'
+            '        <redundancy reply-after="1">1</redundancy>\n'
+            "        <documents>\n"
+            '            <document type="news" mode="index"></document>\n'
+            '            <document type="user" mode="index"></document>\n'
+            "        </documents>\n"
+            "        <nodes>\n"
+            '            <node distribution-key="0" hostalias="node1"></node>\n'
+            "        </nodes>\n"
+            "    </content>\n"
+            "</services>"
+        )
+        self.assertEqual(self.app_package.services_to_text, expected_result)
+
+
 class TestSimplifiedApplicationPackageAddBertRanking(unittest.TestCase):
     def setUp(self) -> None:
         self.app_package = ApplicationPackage(name="test_app")
@@ -939,6 +1095,7 @@ class TestSimplifiedApplicationPackageAddBertRanking(unittest.TestCase):
             first_phase="bm25(title)",
             second_phase=SecondPhaseRanking(rerank_count=10, expression="logit1"),
         )
+        self.disk_folder = "saved_app"
 
     def test_application_package(self):
         self.assertEqual(
@@ -1042,7 +1199,7 @@ class TestSimplifiedApplicationPackageAddBertRanking(unittest.TestCase):
             "    }\n"
             "}"
         )
-        self.assertEqual(self.app_package.schema_to_text, expected_result)
+        self.assertEqual(self.app_package.schema.schema_to_text, expected_result)
 
     def test_hosts_to_text(self):
         expected_result = (
@@ -1096,3 +1253,12 @@ class TestSimplifiedApplicationPackageAddBertRanking(unittest.TestCase):
             "</query-profile-type>"
         )
         self.assertEqual(self.app_package.query_profile_type_to_text, expected_result)
+
+    def test_save_load(self):
+        self.app_package.save(disk_folder=self.disk_folder)
+        self.assertEqual(
+            self.app_package, ApplicationPackage.load(disk_folder=self.disk_folder)
+        )
+
+    def tearDown(self) -> None:
+        rmtree(self.disk_folder, ignore_errors=True)
